# -*- coding: utf-8 -*-
from pathlib import Path

import pytest

from cjk_commons.settings import OrderedDictMergeException, get_settings, merge
from cjk_commons.zip import extract_from_zip, write_to_zip


def test_get_settings_1():
    with pytest.raises(Exception) as e:
        get_settings(app_name='bla', app_author='bla')
        assert e == 'Settings file does not exist'


def test_get_settings_2():
    with pytest.raises(Exception) as e:
        get_settings(Path('bla.yaml'))
        assert e == 'Argument \'app_name\' does not exist'


def test_get_settings_3():
    assert isinstance(get_settings(Path('tests/data/settings.yaml')), dict)


def test_merge_1():
    a = {'a': 1}
    b = {'a': 1}
    assert isinstance(merge(a, b), dict)


def test_merge_2():
    a = {'a': 1}
    b = {'a': 2}
    with pytest.raises(OrderedDictMergeException):
        merge(a, b)


def test_merge_3():
    a = {'b': {'a', 1}}
    b = {'b': {'b', 2}}
    c = merge(a, b)
    assert isinstance(c, dict)


def test_extract_from_zip(tmpdir):
    temp_dir_fullpath = Path(tmpdir)
    extract_from_zip(Path('tests/data/test.zip'), temp_dir_fullpath)
    assert Path(temp_dir_fullpath, 'test.txt').is_file()


def test_write_to_zip_1(tmpdir):
    temp_dir_fullpath = Path(tmpdir)
    write_to_zip(Path(temp_dir_fullpath, 'test.zip'), Path('tests/data/test.txt'))
    assert Path(temp_dir_fullpath, 'test.zip').is_file()


def test_write_to_zip_2(tmpdir):
    temp_dir_fullpath = Path(tmpdir)
    write_to_zip(Path(temp_dir_fullpath, 'test.zip'), Path('tests/data/test'))
    assert Path(temp_dir_fullpath, 'test.zip').is_file()



# -*- coding: utf-8 -*-
from pathlib import Path
from typing import Any

import yaml
import yodl
from appdirs import site_data_dir, user_data_dir
from loguru import logger


def get_attribute(
        kwargs: dict, kwargs_key: str, settings: dict = None, settings_key: str = None,
        default: Any = None, type_: type = str, allow_none: bool = False) -> Any:
    result = None
    type__ = type_
    if default is not None:
        type__ = type(default)
    if kwargs_key in kwargs:
        result = kwargs[kwargs_key]
        if not isinstance(result, type__) and default is not None:
            raise TypeError(f'{kwargs_key}' if kwargs_key else None)
    if settings is not None and settings_key is not None:
        if settings_key in settings:
            result = settings[settings_key]
            if not isinstance(result, type__) and default is not None:
                raise TypeError(f'{settings_key}' if settings_key else None)
    if result is None:
        result = default
    if result is None and not allow_none:
        raise AttributeError(f'{settings_key}')
    return result


def get_path_attribute(
        kwargs: dict, kwargs_key: str, settings: dict = None, settings_key: str = None,
        default_path: Path = None, is_dir: bool = True, check_if_exists: bool = True, create_dir: bool = True,
        create_parents: bool = True) -> Path:
    result = None
    if kwargs_key in kwargs:
        result = kwargs[kwargs_key]
        if not isinstance(result, Path):
            raise TypeError(f'{kwargs_key}' if kwargs_key else None)
    elif settings is not None and settings_key is not None:
        if settings_key in settings:
            result_str = settings[settings_key]
            if not isinstance(result_str, str):
                raise TypeError(f'{settings_key}' if settings_key else None)
            result = Path(result_str)
            if not isinstance(result, Path):
                raise TypeError(f'{settings_key}' if settings_key else None)
    if result is None and isinstance(default_path, Path):
        result = default_path
    if result is None:
        raise AttributeError(f'{settings_key}')
    if result.exists():
        if not is_dir and result.is_dir():
            raise FileExistsError(f'{kwargs_key} Not A File' if kwargs_key else None, result)
        elif is_dir and result.is_file():
            raise NotADirectoryError(f'{kwargs_key}' if kwargs_key else None, result)
    else:
        if check_if_exists:
            raise FileExistsError(f'{kwargs_key}' if kwargs_key else None, result)
        if is_dir and create_dir:
            result.mkdir(parents=create_parents)
    return result


class SettingsError(Exception):
    """Settings Error"""


def get_settings(file_path=Path('settings.yaml'), **kwargs) -> dict:
    if not file_path.is_file():
        app_name = get_attribute(kwargs, 'app_name', allow_none=True)
        app_author = get_attribute(kwargs, 'app_author', allow_none=True)
        file_path = Path(user_data_dir(app_name, app_author, roaming=True), file_path.name)
        if not file_path.is_file():
            file_path = Path(site_data_dir(app_name, app_author), file_path.name)
    if file_path.is_file():
        with file_path.open(encoding='utf-8') as settings_file:
            settings = yaml.load(settings_file, yodl.OrderedDictYAMLLoader)
        if settings is None:
            settings = {}
    else:
        settings = {}
    return settings


class OrderedDictMergeException(Exception):
    """Ordered Dict Merge Exception"""


def merge(a: dict, b: dict, path=None) -> dict:
    if path is None:
        path = []
    for key in b:
        if key in a:
            if isinstance(a[key], dict) and isinstance(b[key], dict):
                merge(a[key], b[key], path + [str(key)])
            elif a[key] == b[key]:
                pass
            else:
                raise OrderedDictMergeException(f'Conflict at \'{".".join(path + [str(key)])}\'')
        else:
            a[key] = b[key]
    return a


logger.disable(__name__)



# -*- coding: utf-8 -*-
import logging
import sys
from pathlib import Path

import loguru


def add_logging_arguments(parser) -> None:
    parser.add_argument(
        '-l', '--level',
        nargs='?',
        help='Set log\'s level'
    )
    parser.add_argument(
        '--log-file',
        nargs='?',
        help='Set log file'
    )
    parser.add_argument(
        '--log-file-level',
        nargs='?',
        help='Set log file\'s level'
    )


def add_loggers(args, logger: loguru.logger, log_file_name: str = '') -> None:
    if args.level is not None:
        level_str = args.level
    else:
        level_str = 'INFO'
    level_str = level_str.upper()
    level_int = getattr(logging, level_str, None)
    if not isinstance(level_int, int):
        raise ValueError(f'Invalid log level \'{level_str}\'')
    logger.remove()
    logger.add(sys.stderr, level=level_str)
    if args.log_file is not None:
        log_file_path = Path(args.log_file)
        if log_file_path.is_dir():
            if Path(log_file_name).stem == log_file_name:
                log_file_name += '.log'
            log_file_path = Path(log_file_path, log_file_name)
        if args.log_file_level is not None:
            log_file_level_str = args.log_file_level
        else:
            log_file_level_str = 'INFO'
        log_file_level_str = log_file_level_str.upper()
        log_file_level_int = getattr(logging, log_file_level_str, None)
        if not isinstance(log_file_level_int, int):
            raise ValueError(f'Invalid log file level \'{log_file_level_str}\'')
        logger.add(log_file_path, level=log_file_level_str)



# -*- coding: utf-8 -*-
import os
from pathlib import Path
import time
from typing import List
import zipfile

from loguru import logger


def extract_from_zip(zip_path: Path, dir_path: Path) -> None:
    with zipfile.ZipFile(zip_path) as zip_file:
        for zip_member in zip_file.infolist():
            zip_file.extract(zip_member, dir_path)
            zip_member_time = time.mktime(zip_member.date_time + (0, 0, -1))
            os.utime(Path(dir_path, zip_member.filename), (zip_member_time, zip_member_time))


def write_to_zip(zip_path: Path, in_path: Path, file_paths: List[Path] = None) -> float:
    with zipfile.ZipFile(zip_path, 'w', zipfile.ZIP_DEFLATED) as zip_file:
        mtime = -1.
        if in_path.is_file():
            zip_file.write(in_path, in_path.name)
            mtime = in_path.stat().st_mtime
        elif in_path.is_dir():
            if not file_paths:
                file_paths = []
                for root, dirnames, filenames in os.walk(str(in_path)):
                    for filename in filenames:
                        file_paths.append(Path(root, filename))
            for file_path in file_paths:
                if file_path.is_file():
                    zip_file.write(file_path, file_path.relative_to(in_path))
                    file_stat_result = file_path.stat()  # todo
                    if mtime < file_stat_result.st_mtime:
                        mtime = file_stat_result.st_mtime
        return mtime


logger.disable(__name__)
